<?php

declare(strict_types=1);

namespace Drupal\Tests\visitors\Kernel\Plugin\Block;

use Drupal\KernelTests\KernelTestBase;
use Symfony\Component\Yaml\Yaml;

require_once __DIR__ . '/../../../visitors.install';

/**
 * Tests the hook_update_30000() function.
 *
 * @group visitors
 */
class HookUpdate30000Test extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected $strictConfigSchema = FALSE;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'user',
    'visitors',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installSchema('visitors', ['visitors_counter']);
    $block = $this->config('visitors.config');
    $block->setData($this->getLegacyConfig());
    $block->save();
  }

  /**
   * Tests the hook_update_30000() function.
   */
  public function testHookUpdate30000() {

    $sandbox = [];

    visitors_update_30000($sandbox);

    $this->assertEquals($sandbox['current'], 1);
    $this->assertEquals($sandbox['max'], 4);
    $this->assertEquals($sandbox['#finished'], 0.25);

    $settings = $this->config('visitors.settings')->getRawData();
    $expected_settings = $this->getLegacyConfig();
    $this->assertEquals($settings, $expected_settings);

    // $legacy_config = $this->config('visitors.config');
    // $this->assertNull($legacy_config);
    visitors_update_30000($sandbox);

    $this->assertEquals($sandbox['current'], 2);
    $this->assertEquals($sandbox['max'], 4);
    $this->assertEquals($sandbox['#finished'], 0.5);

    $schema = \Drupal::database()->schema();
    $this->assertTrue($schema->tableExists('visitors_visit'));
    $this->assertTrue($schema->tableExists('visitors_event'));

    visitors_update_30000($sandbox);

    $this->assertEquals($sandbox['current'], 3);
    $this->assertEquals($sandbox['max'], 4);
    $this->assertEquals($sandbox['#finished'], 0.75);

    visitors_update_30000($sandbox);

    $this->assertEquals($sandbox['current'], 4);
    $this->assertEquals($sandbox['max'], 4);
    $this->assertEquals($sandbox['#finished'], 1);
  }

  /**
   * Tests the settings schema.
   */
  protected function getLegacyConfig(): array {
    $yaml = <<<YAML
flush_log_timer: 0
bot_retention_log: 0
items_per_page: 10
disable_tracking: false
track:
  userid: true
counter:
  enabled: true
  entity_types:
    - 'node'
  display_max_age: 3600
visibility:
  request_path_mode: 0
  request_path_pages: ""
  user_role_mode: 0
  user_role_roles: {}
  user_account_mode: 1
  exclude_user1: false
script_type: minified
YAML;

    $config_array = Yaml::parse($yaml);

    return $config_array;
  }

}
