<?php

namespace Drupal\Tests\visitors\Unit;

use Drupal\Tests\UnitTestCase;
use Symfony\Component\DependencyInjection\ContainerBuilder;

require_once __DIR__ . '/../../../visitors.install';

/**
 * Tests visitors_update_30000().
 *
 * @group visitors
 */
class HookUpdate30000Test extends UnitTestCase {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $configFactory;

  /**
   * The settings config.
   *
   * @var \Drupal\Core\Config\Config|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $settings;

  /**
   * The config.
   *
   * @var \Drupal\Core\Config\Config|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $config;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $messenger;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $moduleHandler;

  /**
   * The module installer.
   *
   * @var \Drupal\Core\Extension\ModuleInstallerInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $moduleInstaller;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $database;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $container = new ContainerBuilder();

    $this->configFactory = $this->createMock('Drupal\Core\Config\ConfigFactoryInterface');
    $container->set('config.factory', $this->configFactory);

    $this->messenger = $this->createMock('Drupal\Core\Messenger\MessengerInterface');
    $container->set('messenger', $this->messenger);

    $this->moduleHandler = $this->createMock('Drupal\Core\Extension\ModuleHandlerInterface');
    $container->set('module_handler', $this->moduleHandler);

    $this->moduleInstaller = $this->createMock('Drupal\Core\Extension\ModuleInstallerInterface');
    $container->set('module_installer', $this->moduleInstaller);

    $this->database = $this->createMock('Drupal\Core\Database\Connection');
    $container->set('database', $this->database);

    \Drupal::setContainer($container);

    $this->settings = $this->createMock('Drupal\Core\Config\Config');
    $this->config = $this->createMock('Drupal\Core\Config\Config');
  }

  /**
   * Tests visitors_update_30000().
   */
  public function testStepZero(): void {
    $this->configFactory->expects($this->exactly(2))
      ->method('getEditable')
      ->willReturnMap([
        ['visitors.settings', $this->settings],
        ['visitors.config', $this->config],
      ]);

    $this->config->expects($this->once())
      ->method('getRawData')
      ->willReturn([]);
    $this->config->expects($this->once())
      ->method('delete');

    $this->settings->expects($this->once())
      ->method('setData')
      ->willReturnSelf();
    $this->settings->expects($this->once())
      ->method('save');

    $sandbox = [];
    visitors_update_30000($sandbox);

    $this->assertEquals(1, $sandbox['current']);
    $this->assertEquals(4, $sandbox['max']);
    $this->assertEquals(0.25, $sandbox['#finished']);
  }

  /**
   * Tests visitors_update_30000().
   */
  public function testStepOne(): void {
    // @todo replace visitors_schema() with static before an initial release.
    $visitors_schema = visitors_schema();
    $schema = $this->createMock('Drupal\Core\Database\Schema');
    $this->database->expects($this->once())
      ->method('schema')
      ->willReturn($schema);
    $schema->expects($this->exactly(2))
      ->method('createTable')
      ->willReturnMap([
        ['visitors_visit', $visitors_schema['visitors_visit']],
        ['visitors_event', $visitors_schema['visitors_event']],
      ]);

    $sandbox = [
      'current' => 1,
      'max' => 4,
    ];
    visitors_update_30000($sandbox);

    $this->assertEquals(2, $sandbox['current']);
    $this->assertEquals(4, $sandbox['max']);
    $this->assertEquals(0.5, $sandbox['#finished']);
  }

  /**
   * Tests visitors_update_30000().
   */
  public function testStepTwo(): void {
    $view = $this->createMock('Drupal\Core\Config\Config');
    $geoip_view = $this->createMock('Drupal\Core\Config\Config');
    $geoip_view->expects($this->once())
      ->method('delete');
    $view->expects($this->once())
      ->method('setData')
      ->willReturnSelf();
    $view->expects($this->once())
      ->method('save');
    $this->configFactory->expects($this->exactly(2))
      ->method('getEditable')
      ->willReturnMap([
        ['views.view.visitors', $view],
        ['views.view.visitors_geoip', $geoip_view],
      ]);

    $sandbox = [
      'current' => 2,
      'max' => 4,
    ];
    visitors_update_30000($sandbox);

    $this->assertEquals(3, $sandbox['current']);
    $this->assertEquals(4, $sandbox['max']);
    $this->assertEquals(0.75, $sandbox['#finished']);
  }

  /**
   * Tests visitors_update_30000().
   */
  public function testStepThree(): void {
    $this->moduleHandler->expects($this->once())
      ->method('moduleExists')
      ->with('visitors_geoip')
      ->willReturn(TRUE);

    $this->moduleInstaller->expects($this->once())
      ->method('uninstall')
      ->with(['visitors_geoip']);

    $sandbox = [
      'current' => 3,
      'max' => 4,
    ];
    visitors_update_30000($sandbox);

    $this->assertEquals(4, $sandbox['current']);
    $this->assertEquals(4, $sandbox['max']);
    $this->assertEquals(1, $sandbox['#finished']);
  }

}
